/*
 * Copyright (c) 2007, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.wso2.registry.app;

import org.apache.abdera.protocol.server.*;
import org.apache.abdera.protocol.server.context.RequestContextWrapper;
import org.apache.abdera.protocol.server.impl.AbstractWorkspaceProvider;
import org.apache.abdera.protocol.server.impl.SimpleWorkspaceInfo;
import org.apache.abdera.protocol.server.impl.TemplateTargetBuilder;
import org.wso2.registry.Registry;
import org.wso2.registry.config.RegistryContext;
import org.wso2.registry.config.RegistryConfigurationProcessor;
import org.wso2.registry.jdbc.InMemoryJDBCRegistry;
import org.wso2.registry.jdbc.JDBCRegistry;
import org.wso2.registry.jdbc.EmbeddedRegistry;
import org.wso2.registry.jdbc.InMemoryEmbeddedRegistry;
import org.wso2.registry.jdbc.realm.InMemoryRegistryRealm;
import org.wso2.registry.jdbc.realm.RegistryRealm;

import java.io.InputStream;
import java.io.FileInputStream;
import java.io.FileNotFoundException;

public class RegistryProvider extends AbstractWorkspaceProvider {
    RegistryAdapter adapter;
    RegistryResolver resolver;
    Registry registry;
    public static final String baseURI = "/wso2registry";

    public class SimpleFilter implements Filter {
        public ResponseContext filter(RequestContext request, FilterChain chain) {
            RequestContextWrapper rcw = new RequestContextWrapper(request);
            rcw.setAttribute("offset", 10);
            rcw.setAttribute("count", 10);
            return chain.next(rcw);
        }
    }

    public CollectionAdapter getCollectionAdapter(RequestContext request) {
        return adapter;
    }

//    private void populate() throws RegistryException {
//        Resource r = registry.newResource();
//        r.setContent("here's some text!");
//        r.setMediaType("text/plain");
//        registry.put("/r1", r);
//
//        Collection c = registry.newCollection();
//        registry.put("/dept", c);
//
//        registry.put("/dept/finance", c);
//
//        registry.put("/dept/finance/myserv", r);
//
//        registry.applyTag("/dept/finance/myserv", "cool");
//    }

    public RegistryProvider() throws Exception {
        RegistryRealm realm = new InMemoryRegistryRealm();
        EmbeddedRegistry embeddedRegistry = new InMemoryEmbeddedRegistry(realm);
        registry = embeddedRegistry.getSystemRegistry();

        RegistryContext ctx = new RegistryContext();
        InputStream is = null;
        try {
            is = new FileInputStream("registry.xml");
        } catch (FileNotFoundException e) {
            // no problemo
        }
        RegistryConfigurationProcessor.populateRegistryConfig(is, ctx);
        registry.setRegistryContext(ctx);

        resolver = new RegistryResolver(registry, baseURI);
        adapter = new RegistryAdapter(resolver, embeddedRegistry, realm);
        
        this.setTargetResolver(resolver);

        this.setTargetBuilder(
                new TemplateTargetBuilder().setTemplate(TargetType.TYPE_COLLECTION,
                                                        "{target_base}/atom/{collection}").
                        setTemplate(RegistryResolver.TAGS_TYPE, "{target_base}/atom/{collection};tags"));
        SimpleWorkspaceInfo workspace = new SimpleWorkspaceInfo();
        workspace.setTitle("A Simple Workspace");
        workspace.addCollection(adapter);
        addWorkspace(workspace);
        addFilter(new SimpleFilter());
    }

    protected ResponseContext processCollection(RequestContext context,
                                                CollectionAdapter adapter) {
        return null;
    }
}
